/*********************************************
DrivePWM.h Module
This is the header file for sending PWM commands to drive motors.

Pin Assignments:
Left Motor: PWM = U0; Break = PP0; DIR = PP1; ThF = PP2;   
Right Motor: PWM = U1; Break = PP3; DIR = PP4; ThF = PP5; 

*********************************************/
#ifndef DrivePWM_H
#define DrivePWM_H

#include <stdio.h>

#include "ME218_E128.h"



/**************************************************
# define:
* Definitions for the DrivePWM library Module
***************************************************/

#define L_MOTOR                   0
#define R_MOTOR                   1       

                              
#define Prescale_CLKA             0x02    // PWM Prescale: /4 
#define LMotor_Period             100     // Left Motor PWM Frequency Period
#define RMotor_Period             100     // Right Motor PWM Frequency Period

#define PWMScale_Target_5kHz      (24000000/4/5000/100/2)
#define PWMScale_Target_2kHz      (24000000/4/2000/100/2)
#define PWMScale_Target_1kHz      (24000000/4/1000/100/2)
#define PWMScale_Target_500Hz     (24000000/4/500/100/2)
#define PWMScale_Target_300Hz     (24000000/4/300/100/2)


#define FORWARD                   0
#define BACKWARD                  1
#define CLOCKWISE                 2
#define COUNTER_CLOCKWISE         3
#define STOP                      4



// Public Function Prototypes
void DrivePWM_Init(void); 

void DrivePWM_SetDutyCycle(unsigned char Motor, unsigned char DutyCycle);

void DriveRobot(unsigned char Movement, unsigned char DC_Left, unsigned char DC_Right);



#endif /* DrivePWM_H */
