/****************************************************************************
 Module
   BallDumpSM.c

 Revision
   1.0.1

 Description
   This is the lower level state machine under the Ball collecting State. This module implements the  
   Gen2 Events and Services Framework.

 Notes:
    States:  
    Timers: Timer2: for internal timeout; 
    Events: ES_ = System events,    EV_ = User defined events
 
 History
 When             Who                What/Why
 --------------   ----------         --------
 02/22/2012       Yushi Yang         Start 1st draft
 02/23/2012       Chris Ploch        Finished 1st draft
****************************************************************************/

/*----------------------------- Include Files -----------------------------*/
/* include header files for this state machine as well as any machines at the
   next lower level in the hierarchy that are sub-machines to this machine
*/
#include "ES_Configure.h"
#include "ES_Framework.h"
#include "BallCollectionSM.h"
#include "BallDumpSM.h"
#include "DrivePWM.h"
#include "SPIFSM.h"
#include "RobotMotionSM.h"
#include <stdio.h>

/*----------------------------- Module Defines ----------------------------*/
// define constants for this machine 
// define timers and timeout time (see header file)



/*---------------------------- Module Functions ---------------------------*/
/* prototypes for private functions for this machine, things like during
   functions, entry & exit functions.They should be functions relevant to the
   behavior of this state machine */


/*---------------------------- Module Variables ---------------------------*/
// everybody needs a state variable, you may need others as well

static BallDumpState CurrentState;
static unsigned char DefenseBin;
static unsigned char AvailableBins[2] = {0,0};
static unsigned char LastBump;

/*------------------------------ Module Code ------------------------------*/
/****************************************************************************
 Function
    RunBallDumpSM

 Parameters
   unsigned char: the event to process

 Returns
   unsigned char: an event to return

 Description
   add your description here
 Notes
   uses nested switch/case to implement the machine.

****************************************************************************/
// make recursive call warning into info
#pragma MESSAGE INFORMATION C1855
ES_Event RunBallDumpSM( ES_Event CurrentEvent )
{
   boolean MakeTransition = False; // assume no transition is being made
   BallDumpState NextState = CurrentState;
   ES_Event ReturnEvent = CurrentEvent; // assume we are not consuming event

   switch ( CurrentState )
   {
        
      case FindingBinToDump:   // If current state is FindingBinToDump (Note: no during function for this state)    
      
         OurAvailableBins();    // Query function OurAvailableBins() to know which bins are available
         
          if ( CurrentEvent.EventType != ES_NO_EVENT )        //If an event is active
                {
   					switch (CurrentEvent.EventType)
   					{
   						case EV_20msBeaconDetected:                               // If this event is 20msBeaconDetected
   						
   						   if(AvailableBins[0]==1)                                // If bin 1 is found to be an available bin 
   						   {
   						      DriveRobot(FORWARD, 100,100);                       // Start driving the robot forward towards the beacon
   						       
   						      DefenseBin = 1;                                     // Set defense bin to be 1
   						      
							      NextState = DrivingForwardToDump;                   // Next State will be DrivingForwardToDump State
							      
						         MakeTransition = True;                              // Mark that we are taking a transition
   						   }
							
								ReturnEvent.EventType = ES_NO_EVENT;                   // Consume this event 
							break;
							
							
							
							case EV_18msBeaconDetected:                              // If this event is 18msBeaconDetected
							
							   if((AvailableBins[0]==2)||(AvailableBins[1]==2))      // If bin 2 is found to be an available bin
   						   {
   						      DriveRobot(FORWARD, 100,100);                      // Start driving the robot forward towards the beacon
   						      
   						      DefenseBin = 2;                                    // Set defense bin to be 2
   						      
							      NextState = DrivingForwardToDump;                  // Next State will be DrivingForwardToDump State
							      
						         MakeTransition = True;                             // Mark that we are taking a transition
   						   }
							   
								ReturnEvent.EventType = ES_NO_EVENT;                  // Consume this event 
							break;
							
							case EV_16msBeaconDetected:                              //  If this event is 16msBeaconDetected
							
							   if((AvailableBins[0]==3)||(AvailableBins[1]==3))      // If bin 3 is found to be an available bin
   						   {
   						      DriveRobot(FORWARD, 100,100);                      // Start driving the robot forward towards the beacon
   						      
   						      DefenseBin = 3;                                    // Set defense bin to be 3
   						      
							      NextState = DrivingForwardToDump;                  // Next State will be DrivingForwardToDump State
							      
						         MakeTransition = True;                             // Mark that we are taking a transition
   						   }
   						   
								ReturnEvent.EventType = ES_NO_EVENT;                  // Consume this event 
							break;
							
							case EV_14msBeaconDetected:                              // If this event is 14msBeaconDetected
							
								if((AvailableBins[0]==4)|| (AvailableBins[1]==4))     // If bin 4 is found to be an available bin
   						   {
   						      DriveRobot(FORWARD, 100,100);                      // Start driving the robot forward towards the beacon
   						      
   						      DefenseBin = 4;                                    // Set defense bin to be 4
   						      
							      NextState = DrivingForwardToDump;                  // Next State will be DrivingForwardToDump State
							      
						         MakeTransition = True;                             // Mark that we are taking a transition
   						   }
							   
								ReturnEvent.EventType = ES_NO_EVENT;                  // Consume this event
							break;
                  }
            }
            
      break;  // break of FindingBinToDump
       
       
      
      
      case DrivingForwardToDump:    // If current state is DrivingForwardToDump (Note: no during function for this state)    
         
         if ( CurrentEvent.EventType != ES_NO_EVENT )        //If an event is active
            {
					switch (CurrentEvent.EventType)
					{          
					
					      case EV_LeftUpperBumpSensor: //If event is LeftUpperBumpSensor, then robot has hit the rotating wall, needs to look for available beacons again
							
                     	DriveRobot(CLOCKWISE, 70, 70);                        // Start rotating the robot clockwise
								
							   NextState = FindingBinToDump;                         // Next State will be FindingBinToDump State
							   
								MakeTransition = True;                                // Mark that we are taking a transition

								ReturnEvent.EventType = ES_NO_EVENT;                  // Consume this event 
						
							break; 
							
							
               		case EV_RightUpperBumpSensor: //If event is RightUpperBumpSensor, then robot has hit the rotating wall, needs to look for available beacons again
					
								DriveRobot(COUNTER_CLOCKWISE, 70, 70);                // Start rotating the robot counter-clockwise
								
							   NextState = FindingBinToDump;                         // Next State will be FindingBinToDump State
							   
								MakeTransition = True;                                // Mark that we are taking a transition

								ReturnEvent.EventType = ES_NO_EVENT;                  // Consume this event 
								
							break; 
							
							
							case EV_LeftFrontBumpSensor: //If event is LeftFrontBumpSensor, then robot has hit the playing field wall
							
							   ES_Timer_InitTimer (BALL_DUMP_TIMER, DUMP_BACKUP);     // Initialize BALL_DUMP_TIMER to DUMP_BACKUP time   
							   
								DriveRobot(BACKWARD, 80, 80);                          // Start backing up the robot
								
								LastBump = LEFT;                                       // Record the left bump sensor to be the last bump sensor triggered
								
							   NextState = DumpBackUp;                                // Next State will be DumpBackUp State
							   
								MakeTransition = True;                                 // Mark that we are taking a transition

								ReturnEvent.EventType = ES_NO_EVENT;                   // Consume this event 
							break; 
							
							case EV_RightFrontBumpSensor: //If event is RightFrontBumpSensor, then robot has hit the playing field wall
					
					         ES_Timer_InitTimer (BALL_DUMP_TIMER, DUMP_BACKUP);     // Initialize BALL_DUMP_TIMER to DUMP_BACKUP time
								
								DriveRobot(BACKWARD, 80, 80);                          // Start backing up the robot
								
								LastBump = RIGHT;                                      // Record the right bump sensor to be the last bump sensor triggered
								
							   NextState = DumpBackUp;                                // Next State will be DumpBackUp State
							   
								MakeTransition = True;                                 // Mark that we are taking a transition

								ReturnEvent.EventType = ES_NO_EVENT;                   // Consume this event 
							break; 
							
						  
               } // end of swithc statement
            } // end of if statement
            
      
      break;  // break of DrivingForwardToDump
      
      
      
      case DumpBackUp:  // If current state is DumpBackUp (Note: no during function for this state)
      
           if ( CurrentEvent.EventType != ES_NO_EVENT )     //If an event is active
            {
					switch (CurrentEvent.EventType)
					{                                          					
							case ES_TIMEOUT: //If event is timeout event 
							   if(LastBump == LEFT)                                   // If the left bump sensor is the last bump sensor triggered  
								{
								   ES_Timer_InitTimer (BALL_DUMP_TIMER, DUMP_ADJUST);  // Reinitialize BALL_DUMP_TIMER to be DUMP_ADJUST time
								   
								   DriveRobot(CLOCKWISE, 70, 70);                      // Start rotating the robot clockwise
								}else if(LastBump == RIGHT)
								{                                                      // Else if the right bump sensor is the last bump sensor triggered 
								
								   ES_Timer_InitTimer (BALL_DUMP_TIMER, DUMP_ADJUST);  // Reinitialize BALL_DUMP_TIMER to be DUMP_ADJUST time
								   
								   DriveRobot(COUNTER_CLOCKWISE, 70, 70);              // Start rotating the robot counter-clockwise
								   
								}
							    
							   ES_Timer_InitTimer (5, DUMP_SWEEP);                    // Initialize timer 5 to record DUMP_SWEEP time, last sweep along the outter radius before dumping
							    
							   NextState = DumpSweep;                                 // Next State will be DumpSwee State
							   
								MakeTransition = True;                                 // Mark that we are taking a transition

								ReturnEvent.EventType = ES_NO_EVENT;                   // Consume this event
							break; 

								
               }   // end switch statement 
            } // end if statement
      
      break;  // break of DumpBackUp 
      
      
      
      
      case DumpSweep:   // If current state is DumpSweep (Note: no during function for this state)
         
         if ( CurrentEvent.EventType != ES_NO_EVENT )        //If an event is active
            {
					switch (CurrentEvent.EventType)
					{                                         
							
							case EV_LeftUpperBumpSensor: //If event is LeftUpperBumpSensor, robot has hit the rotating wall during DumpSweep
							                                                          
								if(LastBump == LEFT)                                   // If the left bump sensor is the last bump sensor triggered   
								{
								   DriveRobot(BACKWARD, 100, 60);                      // Start backing up the robot along the outter radius (Left wheel speed > Right wheel speed)
								   
								}else if(LastBump == RIGHT)                            // If the right bump sensor is the last bump sensor triggered  
								{
								   DriveRobot(BACKWARD, 60, 100);                      // Start backing up the robot along the outter radius (Left wheel speed < Right wheel speed)
								}
							   NextState = FindingTape;                               // Next State will be FindingTape State
							   
								MakeTransition = True;                                 // Mark that we are taking a transition

								ReturnEvent.EventType = ES_NO_EVENT;                   // Consume this event 
							break;
							
					      case EV_RightUpperBumpSensor: //If event is RightUpperBumpSensor, robot has hit the rotating wall during DumpSweep
							
								if(LastBump == LEFT)                                   // If the left bump sensor is the last bump sensor triggered 
								{
								   DriveRobot(BACKWARD, 100, 60);                      // Start backing up the robot along the outter radius (Left wheel speed > Right wheel speed)
								   
								}else if(LastBump == RIGHT)                            // If the right bump sensor is the last bump sensor triggered
								{
								   DriveRobot(BACKWARD, 60, 100);                      // Start backing up the robot along the outter radius (Left wheel speed < Right wheel speed)
								}
							   NextState = FindingTape;                               // Next State will be FindingTape State
							   
								MakeTransition = True;                                 // Mark that we are taking a transition

								ReturnEvent.EventType = ES_NO_EVENT;                   // Consume this event 
							break;
							
							case EV_FrontTwoBumpSensor: //If event is FrontTwoBumpSensor, robot has hit the rotating wall during DumpSweep
							
								if(LastBump == LEFT)                                   // If the left bump sensor is the last bump sensor triggered 
								{
								   DriveRobot(BACKWARD, 100, 60);                      // Start backing up the robot along the outter radius (Left wheel speed > Right wheel speed)
								   
								}else if(LastBump == RIGHT)                            // If the right bump sensor is the last bump sensor triggered
								{
								   DriveRobot(BACKWARD, 60, 100);                      // Start backing up the robot along the outter radius (Left wheel speed < Right wheel speed)
								}
							   NextState = FindingTape;                               // Next State will be FindingTape State
							   
								MakeTransition = True;                                 // Mark that we are taking a transition

								ReturnEvent.EventType = ES_NO_EVENT;                   // Consume this event 
							break;
							
							
							
							case ES_TIMEOUT: //If event is timeout event
								if (CurrentEvent.EventParam == BALL_DUMP_TIMER)       // Only respond to the BALL_DUMP_TIMER event (after DUMP_ADJUST seconds later) 
								{
								   
   								if(LastBump == LEFT)                               // If the left bump sensor is the last bump sensor triggered 
   								{
   								   DriveRobot(FORWARD, 100, 60);                   // Start driving the robot forward on the outter radius (Left wheel speed > Right wheel speed)
   								   
   								}else if(LastBump == RIGHT)                        // If the right bump sensor is the last bump sensor triggered 
   								{
   								   DriveRobot(FORWARD, 60, 100);                   // Start driving the robot forward on the outter radius (Left wheel speed < Right wheel speed)
   								}
								} 
								
								if (CurrentEvent.EventParam == 5)                    //  Only respond to the timer 5 event (after DUMP_SWEEP seconds later)
								{
								   
   								if(LastBump == LEFT)                              // If the left bump sensor is the last bump sensor triggered 
   								{
   								   DriveRobot(BACKWARD, 100, 60);                 // Start driving the robot backward on the outter radius (Left wheel speed > Right wheel speed)
   								   
   								}else if(LastBump == RIGHT)                       // If the right bump sensor is the last bump sensor triggered 
   								{
   								   DriveRobot(BACKWARD, 60, 100);                 // Start driving the robot backward on the outter radius (Left wheel speed < Right wheel speed)
   								}
   								NextState = FindingTape;                          // Next State will be FindingTape State
   								
								   MakeTransition = True;                            // Mark that we are taking a transition

								   ReturnEvent.EventType = ES_NO_EVENT;              // Consume this event 
								}
							break;
							 
					} // end switch statement
            } // end if statement
          
      break; // break of DumpSweep State
      
      
      
       
      case FindingTape: // If current state is FindingTape (Note: no during function for this state)
         
           if ( CurrentEvent.EventType != ES_NO_EVENT )   //If an event is active
            {
					switch (CurrentEvent.EventType)
					{                                          
							
							case EV_FrontTapeSensor: //If event is FrontTapeSensor
								
								if(LastBump == LEFT)                          // If the left bump sensor is the last bump sensor triggered
								{
								   DriveRobot(CLOCKWISE, 100, 100);           // Start rotating the robot clockwise to align up with the tape
								   
								}else if(LastBump == RIGHT)                   // If the right bump sensor is the last bump sensor triggered
								{
								   DriveRobot(COUNTER_CLOCKWISE, 100, 100);   // Start rotating the robot counter-clockwise to align up with the tape
								}
								      
								NextState = AlignTape;                        // Next State will be AlignTape State
								
								MakeTransition = True;                        // Mark that we are taking a transition
								
								ReturnEvent.EventType = ES_NO_EVENT;          // Consume this event 
							break;
						
               } // end switch statement
            } // end break statement
            
      break; // break of FindingTape State
       
      
      case AlignTape:    // If this state is AlignTape  (Note: no during function for this state)   
      
            if ( CurrentEvent.EventType != ES_NO_EVENT )        //If an event is active
            {
					switch (CurrentEvent.EventType)
					{                                          
												
							case EV_BackTapeSensor: // If event is BackTapeSensor
							   ES_Timer_InitTimer (BALL_DUMP_TIMER, DUMP_BACKUP);   // Reinitialize  BALL_DUMP_TIMER to record  DUMP_BACKUP time
							   
								DriveRobot(BACKWARD, 100,100);                       // Start driving the robot backward
								
							   NextState = ReverseDriving;                          // Next State will be ReverseDriving State
							   
						      MakeTransition = True;                               // Mark that we are taking a transition
							   
								ReturnEvent.EventType = ES_NO_EVENT;                 // Consume this event 
							break; 
							
               } // end switch statement
            } // end if statement
            
       break;  // break of AlignTape
       
       
        
       
       case ReverseDriving:    // If this state is ReverseDriving (Note: no during function for this state)   
       
            if ( CurrentEvent.EventType != ES_NO_EVENT )       //If an event is active
            {
					switch (CurrentEvent.EventType)
					{
							case EV_RearTwoBumpSensor: //If event is RearTwoBumpSensor
								
								DriveRobot(STOP, 0, 0);                             // Stop the robot for dump
								
                        PTT &= BIT1LO;                                      // Turn off fan
                        
								ES_Timer_InitTimer (BALL_DUMP_TIMER, DUMP_BALLS);   // Reinitialize timer to wait for dumping all the balls
								            
								NextState = BallDumping;                            // Next State will be BallDumping State
								
								MakeTransition = True;                              // Mark that we are taking a transition
								
								ReturnEvent.EventType = ES_NO_EVENT;                // Consume this event 
							break;  
							
							
							case EV_LeftRearBumpSensor: //If event is LeftRearBumpSensor
								
								DriveRobot(BACKWARD, 0, 100);                       // Start reverse driving only the Right wheel so that both rear bump sensors could be triggered
								
								ReturnEvent.EventType = ES_NO_EVENT;                // Consume this event
							break; 
							
							
							case EV_RightRearBumpSensor: //If event is RightRearBumpSensor
								
								DriveRobot(BACKWARD, 100, 0);                      // Start reverse driving only the Left wheel so that both rear bump sensors could be triggered
							
								ReturnEvent.EventType = ES_NO_EVENT;               // Consume this event 
							break; 
							
					      case ES_TIMEOUT:  // If event is timeout
					         if (CurrentEvent.EventParam == BALL_DUMP_TIMER) // Only respond to the BALL_DUMP_TIMER timeout events
							   {
							   
								   DriveRobot(STOP, 0, 0);                            // Stop the robot
								   
                           PTT &= BIT1LO;                                     // Turn off fan
                           
								   ES_Timer_InitTimer (BALL_DUMP_TIMER, DUMP_BALLS);  // Reinitialize timer to wait for dumping all the balls       
								        
								   NextState = BallDumping;                           // Next State will be BallDumping State
								   
								   MakeTransition = True;                             // Mark that we are taking a transition
						      }
								ReturnEvent.EventType = ES_NO_EVENT;                  // Consume this event
					      
					      
					      break;
                 } // end switch
            } // end if
            
       break;  // break of ReverseDriving
       
       
       
       case BallDumping:    // If this state is BallDumping (Note: no during function for this state) 
        
            if ( CurrentEvent.EventType != ES_NO_EVENT )        //If an event is active
            {
					switch (CurrentEvent.EventType)
					{
							case ES_TIMEOUT: //If event is Timeout 
								if (CurrentEvent.EventParam == BALL_DUMP_TIMER) // If event is BALL_DUMP_TIMER timeout (after DUMP_BALLS seconds later)  
								{
                                                                           
								   DriveRobot(FORWARD,50,50);                     // Start driving the robot forward for jerk motion
								   
								   ES_Timer_InitTimer (5, 200);                   // Reinitialize timer 5 for forward jerk motion time
								   
								   ReturnEvent.EventType = ES_NO_EVENT;           // Consume this event 
								}
								
								if (CurrentEvent.EventParam == 5) // If event is Timer 5 Timeout (after driving forward for jerk)  
								{

								   DriveRobot(BACKWARD,100,100);                 // Start driving the robot backward for jerk motion
								   
								   ES_Timer_InitTimer (6, 300);                  // Initialize timer 6 to record backward jerk motion time
								   
								   
								   ReturnEvent.EventType = ES_NO_EVENT;          // Consume this event 
								  
								}
								
								if (CurrentEvent.EventParam == 6) // if event is Timer 6 Timeout (after driving backward for jerk)  
								{
                            
								   DriveRobot(STOP,0,0);                         // Stop the robot
								   
								   ES_Timer_InitTimer (BALL_DUMP_TIMER, 1000);   // Wait for 2 seconds to go into the defense state 
								   
								   MakeTransition = True;                        // Mark that we are taking a transition
								   
								   ReturnEvent.EventType = EV_FinishDumping;     // Post the event EV_FinishDumping
								   								   
								}
							
								
							break;
							
					} // end switch statement
            } // end if statement
            
       break;  // break of BallDumping
       
  
      
       
   }// End of switch states
   
    //   If we are making a state transition
   if (MakeTransition == True)
    {
       //   Execute exit function for current state
       CurrentEvent.EventType = ES_EXIT;
       RunBallCollectionSM(CurrentEvent);

       //Modify state variable
       CurrentState = NextState; 

       //   Execute entry function for new state
       CurrentEvent.EventType = ES_ENTRY;
       RunBallCollectionSM(CurrentEvent);
     }
     return(ReturnEvent);
}

/****************************************************************************
 Function
     StartBallDumpSM

 Parameters
     Event_t CurrentEvent

 Returns
     None

 Description
     Does any required initialization for this state machine

****************************************************************************/
void StartBallDumpSM ( ES_Event CurrentEvent )
{
   // local variable to get debugger to display the value of CurrentEvent
   ES_Event LocalEvent = CurrentEvent;
   // always start on RotatingToCenter state
   CurrentState = FindingBinToDump;
}


/****************************************************************************
 Function
     QueryBallDumpSM

 Parameters
     None

 Returns
     unsigned char The current state of the Template state machine

 Description
     returns the current state of the Template state machine

****************************************************************************/

BallDumpState QueryBallDumpSM ( void )
{
   return(CurrentState);
}

/***************************************************************************
 private functions
 ***************************************************************************/
 
/****************************************************************************
 Function
     QueryDefenseBin

 Parameters
     None

 Returns
     unsigned char The current DefenseBin

****************************************************************************/
 

unsigned char QueryDefenseBin( void )
{
   return(DefenseBin);
}


/****************************************************************************
 Function
     OurAvailableBins

 Parameters
     None

 Returns
     None
     
 Notes
     Assign the AvailableBins[2] the values to the current bins available
****************************************************************************/
void OurAvailableBins(void)
{
   unsigned int CurrentWallAngle; 
   unsigned char Team;
   
   CurrentWallAngle = (QueryStatus(AR_WALL_ANGLE)*2);    // Query the current wall angle from SPI
   Team = QueryTeam();                                   // Query the team the robot is on
   
   switch(Team)       // switch AvailableBins based on current teams
   {
      case REDTEAM:   // if we are red team
      
         if ((CurrentWallAngle >= ANGLE_H) || (CurrentWallAngle < ANGLE_A))         // if the wall angle is between 329 and 31 degrees
         {
            // Available bins = [1,4]
            AvailableBins[0] = 1;                                                
            AvailableBins[1] = 4;
            
         } else if ((CurrentWallAngle >= ANGLE_A) && (CurrentWallAngle < ANGLE_B))  //  if the wall angle is between 31 and 54 degrees
         {
            //AvailableBins = [1,0];
            
            AvailableBins[0] = 1;
            AvailableBins[1] = 0;
            
         } else if ((CurrentWallAngle >= ANGLE_B) && (CurrentWallAngle < ANGLE_C))  //  if the wall angle is between 54 and 121 degrees
         {
            //AvailableBins = [1,2];
           
            AvailableBins[0] = 1;
            AvailableBins[1] = 2;
            
         } else if ((CurrentWallAngle >= ANGLE_C) && (CurrentWallAngle < ANGLE_D))  //  if the wall angle is between 121 and 151 degrees
         {
            //AvailableBins = [2,0];
            
            AvailableBins[0] = 2;
            AvailableBins[1] = 0;
            
         } else if ((CurrentWallAngle >=ANGLE_D) && (CurrentWallAngle < ANGLE_E))  //  if the wall angle is between 151 and 215 degrees
         {
            //AvailableBins = [2,3];
          
            AvailableBins[0] = 2;
            AvailableBins[1] = 3;
            
         } else if ((CurrentWallAngle >= ANGLE_E) && (CurrentWallAngle < ANGLE_F)) //  if the wall angle is between 215 and 240 degrees
         {
            //AvailableBins = [3,0];
                 AvailableBins[0] = 3;
            AvailableBins[1] = 0; 
            
         } else if ((CurrentWallAngle >= ANGLE_F) && (CurrentWallAngle < ANGLE_G))  //  if the wall angle is between 240 and 303 degrees
         {
            //AvailableBins = [3,4];
            
            AvailableBins[0] = 3;
            AvailableBins[1] = 4;
            
         } else if ((CurrentWallAngle >= ANGLE_G) && (CurrentWallAngle < ANGLE_H))  //  if the wall angle is between 303 and 329 degrees
         {
            //AvailableBins = [4,0];
            
            AvailableBins[0] = 4;
            AvailableBins[1] = 0;
         } 
      
      
      
      break;
      
      case BLUETEAM:  // if we are blue team
      
         if ((CurrentWallAngle >= ANGLE_H) || (CurrentWallAngle < ANGLE_A))         // if the wall angle is between 329 and 31 degrees
         {
            //AvailableBins = [2,3];
            
            AvailableBins[0] = 2;
            AvailableBins[1] = 3;
            
         } else if ((CurrentWallAngle >= ANGLE_A) && (CurrentWallAngle < ANGLE_B))   //  if the wall angle is between 31 and 54 degrees
         {
            //AvailableBins = [3,0];
        
            AvailableBins[0] = 3;
            AvailableBins[1] = 0;
            
         } else if ((CurrentWallAngle >= ANGLE_B) && (CurrentWallAngle < ANGLE_C))   //  if the wall angle is between 54 and 121 degrees
         {
            //AvailableBins = [3,4];
         
            AvailableBins[0] = 3;
            AvailableBins[1] = 4;
            
         } else if ((CurrentWallAngle >= ANGLE_C) && (CurrentWallAngle < ANGLE_D))    //  if the wall angle is between 121 and 151 degrees
         {
            //AvailableBins = [4,0];
            
            AvailableBins[0] = 4;
            AvailableBins[1] = 0;
            
         } else if ((CurrentWallAngle >= ANGLE_D) && (CurrentWallAngle < ANGLE_E))   //  if the wall angle is between 151 and 215 degrees
         {
            //AvailableBins = [1,4];
           
            AvailableBins[0] = 1;
            AvailableBins[1] = 4;
            
         } else if ((CurrentWallAngle >= ANGLE_E) && (CurrentWallAngle < ANGLE_F))   //  if the wall angle is between 215 and 240 degrees
         {
            //AvailableBins = [1,0];
           
            AvailableBins[0] = 1;
            AvailableBins[1] = 0; 
            
         } else if ((CurrentWallAngle >= ANGLE_F) && (CurrentWallAngle < ANGLE_G))    //  if the wall angle is between 240 and 303 degrees
         {
            //AvailableBins = [1,2];
           
            AvailableBins[0] = 1;
            AvailableBins[1] = 2;
            
         } else if ((CurrentWallAngle >= ANGLE_G) && (CurrentWallAngle < ANGLE_H))   //  if the wall angle is between 303 and 329 degrees
         {
            //AvailableBins = [2,0];
          
            AvailableBins[0] = 2;
            AvailableBins[1] = 0;
         } 
      
      break;
   }
   
}
