/****************************************************************************
 Module
   SPIFSM.c

 Revision
   1.0.1

 Description
   This module is the FSM for querying the SPI command. The beacon sensing module is also included in this module

****************************************************************************/
/*----------------------------- Include Files -----------------------------*/
/* include header files for this state machine as well as any machines at the
   next lower level in the hierarchy that are sub-machines to this machine
*/

#include "ES_Configure.h"
#include <stdio.h>
#include <hidef.h>

#include "ME218_E128.h"
#include "S12evec.h"

#include "ES_Framework.h"
#include "ES_Events.h"
#include "ES_PostList.h"

#include "DrivePWM.h"
#include "SPIFSM.h"
#include "RobotMotionSM.h"   

/*----------------------------- Module Defines ----------------------------*/

/*---------------------------- Module Functions ---------------------------*/
/* prototypes for private functions for this machine.They should be functions
   relevant to the behavior of this state machine
*/

/*---------------------------- Module Variables ---------------------------*/
// everybody needs a state variable, you may need others as well.
// type of state variable should match htat of enum in header file
static SPIState_t CurrentState;

// with the introduction of Gen2, we need a module level Priority var as well
static uint8_t MyPriority;

// Last read command byte from command generator
//static unsigned int LastReading;

static unsigned char FieldStatus[6];



/*------------------------------ Module Code ------------------------------*/
/****************************************************************************
 Function
     InitSPIFSM

 Parameters
     uint8_t : the priorty of this service

 Returns
     boolean, False if error in initialization, True otherwise

 Description
     Saves away the priority, sets up the initial transition and does any 
     other required initialization for this state machine

****************************************************************************/
boolean InitSPIFSM ( uint8_t Priority )
{
  ES_Event ThisEvent;

  MyPriority = Priority;
  // put us into the InitSPIState
  CurrentState = InitSPIState;
  // post the initial transition event  
  ThisEvent.EventType = ES_INIT;
  if (ES_PostToService( MyPriority, ThisEvent) == True)
  {
      return True;
  }else
  {
      return False;
  }
}
  

/****************************************************************************
 Function
     PostSPIFSM

 Parameters
     EF_Event ThisEvent , the event to post to the queue

 Returns
     boolean False if the Enqueue operation failed, True otherwise

 Description
     Posts an event to this state machine's queue

****************************************************************************/
boolean PostSPIFSM( ES_Event ThisEvent )
{
  return ES_PostToService( MyPriority, ThisEvent);
} 


/****************************************************************************
 Function
    RunSPIFSM

 Parameters
   ES_Event : the event to process

 Returns
   ES_Event, ES_NO_EVENT if no error ES_ERROR otherwise

 Description
   Send and receives commands to and from the SPI hardware
 Notes
   uses nested switch/case to implement the machine.

****************************************************************************/
ES_Event RunSPIFSM( ES_Event ThisEvent )
{
  static SPIState_t NextState;
  static unsigned char LastReading;              // Define the last SPI reading
  static boolean GameOn = False;
  
  unsigned char NewReading;                      // Define the new SPI reading

  unsigned char StateReading;                    // Define the state reading
  
  static unsigned char NewStateReading;          // Define the new state reading
  
  static unsigned char CommandArray[6] = {0xFD /* balls in play*/,0xFC /* Bin 1 balls */,0xFB /* Bin 2 balls */,
                                   0xFA /* Bin 3 balls */,0xF9 /* Bin 4 balls */,0xF8 /* Wall Angle */}; 
    
  static unsigned char i = 0;                   // Assign counter for command sending 
  static unsigned char ArrayNumber;             // Assign array number 
  
  ES_Event GameOnEvent; 
  ES_Event ReturnEvent;
  ReturnEvent.EventType = ES_NO_EVENT;          // Assume no errors
  
  
  switch ( CurrentState )
  {
      case InitSPIState :       // If current state is InitSPIState
          if ( ThisEvent.EventType == ES_INIT ) // only respond to EF_Init
          {
             // Initialize FULL_READ_TIMER to  SENDRECEIVE_TIME 
             ES_Timer_InitTimer(FULL_READ_TIMER, SENDRECEIVE_TIME);             
                  
             // now put the machine into the SendRecieve State
             NextState = SendReceive;
          }
      break;

      
      case SendReceive :      // If current State is  SendReceive          
          switch ( ThisEvent.EventType )
          {
              case ES_TIMEOUT :
              {
                  // If FULL_READ_TIMER has expired
                  if (ThisEvent.EventParam == FULL_READ_TIMER)
                  {                                                              
                     
                     // If transmit register is empty
                     if ((SPISR & _S12_SPTEF) == _S12_SPTEF)
                     {
                             // Transmit data (query command) to master transmit register
                             ArrayNumber = i%6;                              
                              
                                                          
                              switch (ArrayNumber) // switch based on the array number to send different commands
                              {
                                  case 0://AR_TOTAL_BALLS_ONE_SIDE:
                                  {
                                     SPIDR = CommandArray[ArrayNumber];
                                     printf("Command sent is: %x \n\r", CommandArray[ArrayNumber]);
                                     
                                  }
                                  break;
                              
                              
                                  case 1://AR_BIN_1:
                                  {
                                    SPIDR = CommandArray[ArrayNumber];
                                    printf("Command sent is: %x \n\r", CommandArray[ArrayNumber]);
                                    
                                  }
                                  break;
                              
                              
                                  case 2://AR_BIN_2:
                                  {
                                    SPIDR = CommandArray[ArrayNumber];
                                    printf("Command sent is: %x \n\r", CommandArray[ArrayNumber]); 
                                    
                                  }
                                 break;
                              
                              
                                 case 3://AR_BIN_3:
                                 {
                                    SPIDR = CommandArray[ArrayNumber];
                                    printf("Command sent is: %x \n\r", CommandArray[ArrayNumber]);
                                    
                                 }
                                 break;
                              
                              
                                 case 4://AR_BIN_4:
                                 {
                                    SPIDR = CommandArray[ArrayNumber];
                                    printf("Command sent is: %x \n\r", CommandArray[ArrayNumber]);
                                     
                                 }
                                 break;
                              
                                 case 5://AR_WALL_ANGLE:
                                 {
                                    SPIDR = CommandArray[ArrayNumber];
                                    printf("Command sent is: %x \n\r", CommandArray[ArrayNumber]);
                                     
                                 }
                                 break;
                                                            

                                                                         
                              }
                              
                              
                              if((SPISR & _S12_SPIF) == _S12_SPIF)         ///After sending the command, read the junk value
                              {                                                                                 
                                 // Read receive register
                                 NewReading = SPIDR;
                                 
                              }                                                   
                              
                                                        
                                        
                              ES_Timer_InitTimer(BYTE_DELAY_TIMER, _2MS_);  // Re-initialize BYTE_DELAY_TIMER to send out 0x00 to get real data  
                              
 
                     }

                       
                  }else if (ThisEvent.EventParam == BYTE_DELAY_TIMER)
                  {                     
                      // If transmit register is empty (it will be after the 2ms timer has expired)
                      if ((SPISR & _S12_SPTEF) == _S12_SPTEF)
                      {
                          // Send command 0x00
                          SPIDR = 0x00;
                          
                        
                          ES_Timer_InitTimer(RECEIVE_DELAY_TIMER, _2MS_);  // Re-initialize BYTE_DELAY_TIMER to read the real data recieved  
                        
                      } 
                  } 
                  
                  else if (ThisEvent.EventParam == RECEIVE_DELAY_TIMER)
                  {
                       
                      
                      if((SPISR & _S12_SPIF) == _S12_SPIF)
                        {                                                                                     
                           // Read receive register
                           NewReading = SPIDR;                        
                                                      
                           // NOTE: SPIF is cleared with a read of SPISR, with SPIF = 1,
                           // and a read of SPIDR (which occurred above)
                           
                           
                          NewStateReading = NewReading;                   // Assign NewReading to NewStateReading
                            if(ArrayNumber == AR_TOTAL_BALLS_ONE_SIDE)    // If Array number is 0, AR_TOTAL_BALLS_ONE_SIDE 
                           {
                              if (((NewReading-0x80) != 0))    // If current number of balls in the playing field is not 0
                              {
                             
                                 if ((LastReading-0x80) == 0)   // If last number of balls in the playing field is 0
                                 {
                                    // Post event: GameOn
                                    GameOn = True;
                                    GameOnEvent.EventType = EV_GameOn;
                                    PostRobotMotionSM(GameOnEvent);
                                 } // end if
                                                                                                          
                              } // end if
                              LastReading = NewReading;                  // Assign LastRading to NewReading
                              StateReading = NewStateReading - 0x80;     // Assign StateReading to  NewStateReading - 0x80
                                  
                           }else
                           {
                              StateReading = NewStateReading - 1;       // Else, Assign StateReading to  NewStateReading - 1
                           }
                           
                           FieldStatus[ArrayNumber] = StateReading;     // Record StateReading to FieldStatus[] based on the current array number  
                                           

                        } 
                        ES_Timer_InitTimer(FULL_READ_TIMER, SENDRECEIVE_TIME);   // Re-initialize FULL_READ_TIMER to send and recieve the next command
                        
                        i++;                                                     // counter = counter +1
                        
                  }
                                  
                  else   // Read a bogus timeout (not from Timer0)
                  {
                     ReturnEvent.EventType =   ES_ERROR;                    
                  }
                  
              }
              break;               
            
                            
          } // end switch on ThisEvent.EventType in SendReceive state
          
      break;
                       

  }   // end switch on Current State
  
  CurrentState = NextState;
  return ReturnEvent;    
  
}



/****************************************************************************
 Function
     QueryLab8FSM

 Parameters
     None

 Returns
     Lab8State_t The current state of the Lab8 state machine

 Description
     returns the current state of the Lab8 state machine

****************************************************************************/
SPIState_t QuerySPIFSM ( void )
{
   return(CurrentState);
}


/*---------------------------- Public Functions ---------------------------*/

/****************************************************************************
 Function
     InitSPI(void)

 Parameters
     None

 Returns
     None

 Description
     Initializes SPI communication on-board E128
****************************************************************************/
void InitSPI(void)
{
   // Set baud rate divisor to 0x77, the slowest baud rate
   SPIBR = 0x77;

   // Set clock to idle low (CPOL = 0) and odd edge (CPHA = 0)
   SPICR1 = 0x5E;    // Bit 7: SPIE, SPI interrupt enable           = 0
                     // Bit 6: SPE, SPI system enable bit           = 1
                     // Bit 5: SPTIE, SPI transmit interrupt enable = 0
                     // Bit 4: MSTR, SPI master/slave mode select   = 1
                     // Bit 3: CPOL, SPI clock polarity             = 1 (idle high)
                     // Bit 2: CPHA, SPI clock phase                = 1 (even edges)
                     // Bit 1: SSOE, Slave select output enable     = 1 
                     // Bit 0: LSBFE, LSB first enable              = 0 (MSB first)       
   
   // Set MODFEN, mode fault enable, high
   SPICR2 = 0x10;
   //printf("Finish SPI Initialization \n\r");   
}


/****************************************************************************
 Function
     QueryState

 Parameters
     None

 Returns
     unsigned char - the new command byte from the command generator

****************************************************************************/
unsigned char QueryStatus(unsigned char DesiredStatus)
{
   return (FieldStatus[DesiredStatus]);
}


/***************************************************************************
 BeaconSensing Module

****************************************************************************/

/****************************************************************************
 Function
     InitBeaconDetectionTimer()

 Parameters
     None

 Returns
     None

 Description
     Initialize timer for beacon sensing Input Capture
****************************************************************************/
void InitBeaconDetectionTimer(void)
{     
   // Set pin T0 as in input (Use TIM0_CLK4)
   DDRT &= BIT0LO;  
  
   // Enable the timer system
   TIM0_TSCR1 = _S12_TEN;                           
   
   // Set prescale to M/128
   TIM0_TSCR2 |= (_S12_PR2 | _S12_PR1 | _S12_PR0) ; 
    
   // Settings for input capture
   
   TIM0_TIOS &= (~_S12_IOS4);    // Sets PT0 to IC4 (set bit = 0)
   TIM0_TIE |= _S12_C4I;         // Enable IC4 interrupt (PT0)
   TIM0_TCTL3 |= (_S12_EDG4A);   // Capture rising edges
   TIM0_TFLG1 = _S12_C4F;        // Clear IC4 flag  

   EnableInterrupts;             // Enable global interrupts
}  


/****************************************************************************
 Function
     interrupt _Vec_tim0ch4 BeaconInputCapture()

 Parameters
     None

 Returns
     None

 Description
     Input Capture function for capturing IR beacon signal
****************************************************************************/     
void interrupt _Vec_tim0ch4 BeaconInputCapture(void)
{
   static unsigned int uLastEdge =0; 
   static unsigned int uPeriod;     
   
      
   // Store period in local variable
   uPeriod = TIM0_TC4 - uLastEdge;
   
   // Update last edge reading
   uLastEdge = TIM0_TC4;
   
   // Clear interrupt flag
   TIM0_TFLG1 = _S12_C4F; 
   
   // Post events based on uPeriod 
  if ((uPeriod < MAX_PERIOD_20 )&&(uPeriod > MIN_PERIOD_20))
   {
      // Post 20ms beacon detected event
      ES_Event ThisEvent;
      ThisEvent.EventType = EV_20msBeaconDetected;
      ES_PostList01(ThisEvent);
             
   }else if ((uPeriod < MAX_PERIOD_18 )&&(uPeriod > MIN_PERIOD_18))
   {
      
      // Post 18ms beacon detected event
      ES_Event ThisEvent;
      ThisEvent.EventType = EV_18msBeaconDetected;
      ES_PostList01(ThisEvent);
        
   }else if ((uPeriod < MAX_PERIOD_16)&&(uPeriod > MIN_PERIOD_16))
   {
      // Post 16ms beacon detected event
      ES_Event ThisEvent;
      ThisEvent.EventType = EV_16msBeaconDetected;
      ES_PostList01(ThisEvent);
        
   }else if ((uPeriod < MAX_PERIOD_14)&&(uPeriod > MIN_PERIOD_14))
   {
      // Post 14ms beacon detected event 
      ES_Event ThisEvent;
      ThisEvent.EventType = EV_14msBeaconDetected;
      ES_PostList01(ThisEvent);
   }  
   
} 

/*------------------------------ End of file ------------------------------*/

